"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dts_critic_1 = require("dts-critic");
const Lint = require("tslint");
const util_1 = require("../util");
const defaultOptions = {
    mode: dts_critic_1.Mode.NameOnly,
};
class Rule extends Lint.Rules.AbstractRule {
    apply(sourceFile) {
        return this.applyWithFunction(sourceFile, walk, parseOptions(this.ruleArguments));
    }
}
exports.Rule = Rule;
Rule.metadata = {
    ruleName: "npm-naming",
    description: "Ensure that package name and DefinitelyTyped header match npm package info.",
    optionsDescription: `An object with a \`mode\` property should be provided.
If \`mode\` is '${dts_critic_1.Mode.Code}', then option \`errors\` can be provided.
\`errors\` should be an array specifying which code checks should be enabled or disabled.`,
    options: {
        oneOf: [
            {
                type: "object",
                properties: {
                    "mode": {
                        type: "string",
                        enum: [dts_critic_1.Mode.NameOnly],
                    },
                    "single-line": {
                        description: "Whether to print error messages in a single line. Used for testing.",
                        type: "boolean",
                    },
                    "required": ["mode"],
                },
            },
            {
                type: "object",
                properties: {
                    "mode": {
                        type: "string",
                        enum: [dts_critic_1.Mode.Code],
                    },
                    "errors": {
                        type: "array",
                        items: {
                            type: "array",
                            items: [
                                { description: "Name of the check.",
                                    type: "string",
                                    enum: [dts_critic_1.ErrorKind.NeedsExportEquals, dts_critic_1.ErrorKind.NoDefaultExport],
                                },
                                {
                                    description: "Whether the check is enabled or disabled.",
                                    type: "boolean",
                                },
                            ],
                            minItems: 2,
                            maxItems: 2,
                        },
                    },
                    "single-line": {
                        description: "Whether to print error messages in a single line. Used for testing.",
                        type: "boolean",
                    },
                    "required": ["mode"],
                },
            },
        ],
    },
    optionExamples: [
        true,
        [true, { mode: dts_critic_1.Mode.NameOnly }],
        [true, { mode: dts_critic_1.Mode.Code, errors: [[dts_critic_1.ErrorKind.NeedsExportEquals, true], [dts_critic_1.ErrorKind.NoDefaultExport, false]] }],
    ],
    type: "functionality",
    typescriptOnly: true,
};
function parseOptions(args) {
    if (args.length === 0) {
        return defaultOptions;
    }
    const arg = args[0];
    if (arg == null) {
        return defaultOptions;
    }
    if (!arg.mode || typeof arg.mode !== "string") {
        return defaultOptions;
    }
    const mode = dts_critic_1.parseMode(arg.mode);
    if (!mode) {
        return defaultOptions;
    }
    const singleLine = !!arg["single-line"];
    switch (mode) {
        case dts_critic_1.Mode.NameOnly:
            return { mode, singleLine };
        case dts_critic_1.Mode.Code:
            if (!arg.errors || !Array.isArray(arg.errors)) {
                return { mode, errors: new Map(), singleLine };
            }
            return { mode, errors: parseEnabledErrors(arg.errors), singleLine };
    }
}
function parseEnabledErrors(errors) {
    const enabledChecks = new Map();
    for (const tuple of errors) {
        if (Array.isArray(tuple)
            && tuple.length === 2
            && typeof tuple[0] === "string"
            && typeof tuple[1] === "boolean") {
            const error = dts_critic_1.parseExportErrorKind(tuple[0]);
            if (error) {
                enabledChecks.set(error, tuple[1]);
            }
        }
    }
    return enabledChecks;
}
function tslintDisableOption(error) {
    switch (error) {
        case dts_critic_1.ErrorKind.NoMatchingNpmPackage:
        case dts_critic_1.ErrorKind.NoMatchingNpmVersion:
        case dts_critic_1.ErrorKind.NonNpmHasMatchingPackage:
            return `false`;
        case dts_critic_1.ErrorKind.NoDefaultExport:
        case dts_critic_1.ErrorKind.NeedsExportEquals:
        case dts_critic_1.ErrorKind.JsSignatureNotInDts:
        case dts_critic_1.ErrorKind.JsPropertyNotInDts:
        case dts_critic_1.ErrorKind.DtsSignatureNotInJs:
        case dts_critic_1.ErrorKind.DtsPropertyNotInJs:
            return JSON.stringify([true, { mode: dts_critic_1.Mode.Code, errors: [[error, false]] }]);
    }
}
function errorMessage(error, opts) {
    const message = error.message +
        `\nIf you won't fix this error now or you think this error is wrong,
you can disable this check by adding the following options to your project's tslint.json file under "rules":

    "npm-naming": ${tslintDisableOption(error.kind)}
`;
    if (opts.singleLine) {
        return message.replace(/(\r\n|\n|\r|\t)/gm, " ");
    }
    return message;
}
function walk(ctx) {
    const { sourceFile } = ctx;
    const { text } = sourceFile;
    const lookFor = (search, explanation) => {
        const idx = text.indexOf(search);
        if (idx !== -1) {
            ctx.addFailureAt(idx, search.length, util_1.failure(Rule.metadata.ruleName, explanation));
        }
    };
    if (util_1.isMainFile(sourceFile.fileName, /*allowNested*/ false)) {
        try {
            const errors = dts_critic_1.dtsCritic(sourceFile.fileName, /* sourcePath */ undefined, ctx.options);
            for (const error of errors) {
                switch (error.kind) {
                    case dts_critic_1.ErrorKind.NoMatchingNpmPackage:
                    case dts_critic_1.ErrorKind.NoMatchingNpmVersion:
                    case dts_critic_1.ErrorKind.NonNpmHasMatchingPackage:
                        lookFor("// Type definitions for", errorMessage(error, ctx.options));
                        break;
                    case dts_critic_1.ErrorKind.DtsPropertyNotInJs:
                    case dts_critic_1.ErrorKind.DtsSignatureNotInJs:
                    case dts_critic_1.ErrorKind.JsPropertyNotInDts:
                    case dts_critic_1.ErrorKind.JsSignatureNotInDts:
                    case dts_critic_1.ErrorKind.NeedsExportEquals:
                    case dts_critic_1.ErrorKind.NoDefaultExport:
                        if (error.position) {
                            ctx.addFailureAt(error.position.start, error.position.length, util_1.failure(Rule.metadata.ruleName, errorMessage(error, ctx.options)));
                        }
                        else {
                            ctx.addFailure(0, 1, util_1.failure(Rule.metadata.ruleName, errorMessage(error, ctx.options)));
                        }
                        break;
                }
            }
        }
        catch (e) {
            // We're ignoring exceptions.
        }
    }
    // Don't recur, we're done.
}
//# sourceMappingURL=npmNamingRule.js.map